@@ -2,4 +2,6 @@ from .qpixelsize import *
 from .origin import *
 from .ellipse import *
 from .rotation import *
+from .probe import *
+from .braggvectors import *
 
