@@ -1,5 +1,6 @@
 import warnings
 import numpy as np
+import numpy.lib.recfunctions as rfn
 from scipy import linalg
 from typing import Union, Optional, Dict, Tuple, List
 from time import time
@@ -8,7 +9,6 @@ from dataclasses import dataclass
 
 from ...io.datastructure import PointList
 from ..utils import electron_wavelength_angstrom, single_atom_scatter
-from ..dpc import get_interaction_constant
 from .WK_scattering_factors import compute_WK_factor
 
 
@@ -23,7 +23,7 @@ def calculate_dynamical_structure_factors(
     accelerating_voltage: float,
     method: str = "WK-CP",
     k_max: float = 2.0,
-    thermal_sigma: float = None,
+    thermal_sigma: Optional[Union[float, dict]] = None,
     tol_structure_factor: float = 1.0e-4,
     verbose=True,
 ):
@@ -51,10 +51,12 @@ def calculate_dynamical_structure_factors(
         k_max (float):                  max scattering length to compute structure factors to.
                                         Setting this to 2x the k_max used in generating the beamsn
                                         included in a simulation will retain all possible couplings
-        thermal_sigma (float):  RMS atomic diplacement for attenuating form factors to account for thermal
+        thermal_sigma (float or dict{int->float}):  RMS atomic diplacement for attenuating form factors to account for thermal
                                         broadening of the potential, only used when a "WK" method is
                                         selected. Required when WK-P or WK-CP are selected.
                                         Units are Å. (This is often written as 〈u〉in papers)
+                                        To specify different 〈u〉 for each element, pass a dictionary
+                                        with Z as the key, mapping to the appropriate float value
         tol_structure_factor (float):   tolerance for removing low-valued structure factors. Reflections
                                         with structure factor below the tolerance will have zero coupling
                                         in the dynamical calculations (i.e. they are the ignored weak beams)
@@ -71,6 +73,11 @@ def calculate_dynamical_structure_factors(
         "WK-CP",
     ), "Invalid method specified."
 
+    if "WK" in method:
+        assert (
+            thermal_sigma is not None
+        ), "thermal_sigma must be specifed when using W-K potentials"
+
     # Calculate the reciprocal lattice points to include based on k_max
 
     k_max = np.asarray(k_max)
@@ -116,15 +123,15 @@ def calculate_dynamical_structure_factors(
 
     lobato_lookup = single_atom_scatter()
 
-    m0c2 = 5.109989461e5    # electron rest mass, in eV
-    relativistic_factor = (m0c2 + accelerating_voltage)/m0c2
+    m0c2 = 5.109989461e5  # electron rest mass, in eV
+    relativistic_factor = (m0c2 + accelerating_voltage) / m0c2
 
     from functools import lru_cache
 
     # get_f_e returns f^e in units of VÅ^3, with relativistic correction
-    # but not yet converted to 
+    # but not yet converted to
     @lru_cache(maxsize=2 ** 12)
-    def get_f_e(q, Z, B, method):
+    def get_f_e(q, Z, thermal_sigma, method):
         if method == "Lobato":
             # Real lobato factors
             lobato_lookup.get_scattering_factor([Z], [1.0], [q], units="A")
@@ -132,7 +139,9 @@ def calculate_dynamical_structure_factors(
         elif method == "Lobato-absorptive":
             # Fake absorptive Lobato factors
             lobato_lookup.get_scattering_factor([Z], [1.0], [q], units="A")
-            return np.complex128(relativistic_factor / np.pi * lobato_lookup.fe * (1.0 + 0.1j))
+            return np.complex128(
+                relativistic_factor / np.pi * lobato_lookup.fe * (1.0 + 0.1j)
+            )
         elif method == "WK":
             # Real WK factor
             return compute_WK_factor(
@@ -190,20 +199,26 @@ def calculate_dynamical_structure_factors(
         desc=f"Computing {display_names[method]} lookup table",
         disable=not verbose,
     ):
-        Freal = 0.0
-        Fimag = 0.0
+        Fscatt = 0.0 + 0.0j
         for i_pos in range(self.positions.shape[0]):
             # Get the appropriate atomic form factor:
-            fe = get_f_e(g_vec_leng[i_hkl], self.numbers[i_pos], thermal_sigma, method)
+            sigma = (
+                thermal_sigma[self.numbers[i_pos]]
+                if isinstance(thermal_sigma, dict)
+                else thermal_sigma
+            )
+            fe = get_f_e(
+                q=g_vec_leng[i_hkl],
+                Z=self.numbers[i_pos],
+                thermal_sigma=sigma,
+                method=method,
+            )
 
-            # accumulate the real and imag portions separately (?)
-            Freal += np.real(fe) * np.exp(
+            Fscatt += fe * np.exp(
                 (2.0j * np.pi) * (hkl[:, i_hkl] @ self.positions[i_pos])
             )
-            Fimag += np.imag(fe) * np.exp(
-                (2.0j * np.pi) * (hkl[:, i_hkl] @ self.positions[i_pos])
-            )
-        struct_factors[i_hkl] = Freal + 1.0j * Fimag
+
+        struct_factors[i_hkl] = Fscatt
 
     # Divide by unit cell volume
     unit_cell_volume = np.abs(np.linalg.det(self.lat_real))
@@ -242,6 +257,9 @@ def generate_dynamical_diffraction_pattern(
     verbose: bool = False,
     always_return_list: bool = False,
     dynamical_matrix_cache: Optional[DynamicalMatrixCache] = None,
+    return_complex: bool = False,
+    return_eigenvectors: bool = False,
+    return_Smatrix: bool = False,
 ) -> Union[PointList, List[PointList]]:
     """
     Generate a dynamical diffraction pattern (or thickness series of patterns)
@@ -276,12 +294,21 @@ def generate_dynamical_diffraction_pattern(
                                         computed and stored. Subsequent calls will use the cached matrix
                                         for the off-diagonal components of the A matrix and overwrite
                                         the diagonal elements. This is used for CBED calculations.
-
+        return_complex (bool):          When True, returns both the complex amplitude and intensity. Defaults to (False)
     Returns:
         bragg_peaks (PointList):         Bragg peaks with fields [qx, qy, intensity, h, k, l]
             or
         [bragg_peaks,...] (PointList):   If thickness is a list/array, or always_return_list is True,
                                         a list of PointLists is returned.
+        if return_complex = True:
+            bragg_peaks (PointList):         Bragg peaks with fields [qx, qy, intensity, amplitude, h, k, l]
+                or
+            [bragg_peaks,...] (PointList):   If thickness is a list/array, or always_return_list is True,
+                                            a list of PointLists is returned.
+        if return_Smatrix = True:
+            [S_matrix, ...], psi_0:     Returns a list of S-matrices for each thickness (this is always a list),
+                                        and the vector representing the incident plane wave. The beams of the
+                                        S-matrix have the same order as in the input `beams`.
 
     """
     t0 = time()  # start timer for matrix setup
@@ -291,15 +318,18 @@ def generate_dynamical_diffraction_pattern(
     beam_g, beam_h = np.meshgrid(np.arange(n_beams), np.arange(n_beams))
 
     # Parse input orientations:
-    zone_axis_rotation_matrix = self.parse_orientation(zone_axis_lattice=zone_axis_lattice, 
-                                       zone_axis_cartesian=zone_axis_cartesian)
+    zone_axis_rotation_matrix = self.parse_orientation(
+        zone_axis_lattice=zone_axis_lattice, zone_axis_cartesian=zone_axis_cartesian
+    )
     if foil_normal_lattice is not None or foil_normal_cartesian is not None:
-        foil_normal = self.parse_orientation(zone_axis_lattice=foil_normal_lattice,
-                                             zone_axis_cartesian=foil_normal_cartesian)
+        foil_normal = self.parse_orientation(
+            zone_axis_lattice=foil_normal_lattice,
+            zone_axis_cartesian=foil_normal_cartesian,
+        )
     else:
         foil_normal = zone_axis_rotation_matrix
 
-    foil_normal = foil_normal[:,2]
+    foil_normal = foil_normal[:, 2]
 
     # Note the difference in notation versus kinematic function:
     # k0 is the scalar magnitude of the wavevector, rather than
@@ -350,18 +380,9 @@ def generate_dynamical_diffraction_pattern(
 
     # Compute the diagonal entries of \hat{A}: 2 k_0 s_g [5.51]
     g = (hkl @ self.lat_inv) @ zone_axis_rotation_matrix
-    sg = self.excitation_errors(g.T, foil_normal=-foil_normal @ zone_axis_rotation_matrix)
-
-    # import matplotlib.pyplot as plt
-    # sgp = np.sign(sg) >= 0
-    # c = np.zeros_like(g)
-    # c[sgp,:] = np.array([1,0,0])
-    # c[~sgp,:] = np.array([0,0,1])
-    # fig,ax = plt.subplots(dpi=200)
-    # ax.scatter(g[:,0],g[:,1],np.abs(sg)*100,c=c)
-    # ax.axis('equal')
-    # plt.show()
-
+    sg = self.excitation_errors(
+        g.T, foil_normal=-foil_normal @ zone_axis_rotation_matrix
+    )
 
     # Fill in the diagonal, completing the structure mattrx
     np.fill_diagonal(U_gmh, 2 * k0 * sg + 1.0j * np.imag(self.Ug_dict[(0, 0, 0)]))
@@ -377,7 +398,9 @@ def generate_dynamical_diffraction_pattern(
     t0 = time()  # start timer for eigendecomposition
 
     v, C = linalg.eig(U_gmh)  # decompose!
-    gamma = v / (2.0 * k0 * zone_axis_rotation_matrix[:,2] @ foil_normal)  # divide by 2 k_n
+    gamma = v / (
+        2.0 * k0 * zone_axis_rotation_matrix[:, 2] @ foil_normal
+    )  # divide by 2 k_n
 
     # precompute the inverse of C
     C_inv = np.linalg.inv(C)
@@ -395,27 +418,55 @@ def generate_dynamical_diffraction_pattern(
     psi_0 = np.zeros((n_beams,))
     psi_0[int(np.where((hkl == [0, 0, 0]).all(axis=1))[0])] = 1.0
 
-    # calculate the diffraction intensities for each thichness matrix
+    # calculate the diffraction intensities (and amplitudes) for each thichness matrix
     # I = |psi|^2 ; psi = C @ E(z) @ C^-1 @ psi_0, where E(z) is the thickness matrix
-    intensities = [
-        np.abs(C @ (np.exp(2.0j * np.pi * z * gamma) * (C_inv @ psi_0))) ** 2
-        for z in np.atleast_1d(thickness)
-    ]
+    if return_Smatrix:
+        Smatrices = [
+            C @ np.diag(np.exp(2.0j * np.pi * z * gamma)) @ C_inv
+            for z in np.atleast_1d(thickness)
+        ]
+        return Smatrices, psi_0
+    elif return_complex:
+        # calculate the amplitudes
+        amplitudes = [
+            C @ (np.exp(2.0j * np.pi * z * gamma) * (C_inv @ psi_0))
+            for z in np.atleast_1d(thickness)
+        ]
+
+        # Do this first to avoid handling structured array
+        intensities = np.abs(amplitudes) ** 2
+
+        # convert amplitudes as a structured array
+        # do we want complex64 or complex 32.
+        amplitudes = np.array(amplitudes, dtype=([("amplitude", "<c16")]))
+    else:
+        intensities = [
+            np.abs(C @ (np.exp(2.0j * np.pi * z * gamma) * (C_inv @ psi_0))) ** 2
+            for z in np.atleast_1d(thickness)
+        ]
 
     # make new pointlists for each thickness case and copy intensities
     pls = []
     for i in range(len(intensities)):
         newpl = beams.copy()
-        newpl.data["intensity"] = intensities[i]
+        if return_complex:
+            # overwrite the kinematical intensities with the dynamical intensities
+            newpl.data["intensity"] = intensities[i]
+            # merge amplitudes into the list
+            newpl.data = rfn.merge_arrays(
+                (newpl.data, amplitudes[i]), asrecarray=False, flatten=True
+            )
+        else:
+            newpl.data["intensity"] = intensities[i]
         pls.append(newpl)
 
     if verbose:
         print(f"Assembling outputs took {1000*(time()-t0):.3f} ms.")
 
     if len(pls) == 1 and not always_return_list:
-        return pls[0]
+        return pls[0] if not return_eigenvectors else (pls[0], gamma, C)
     else:
-        return pls
+        return pls if not return_eigenvectors else (pls, gamma, C)
 
 
 def generate_CBED(
@@ -435,6 +486,7 @@ def generate_CBED(
     progress_bar: bool = True,
     return_mask: bool = False,
     two_beam_zone_axis_lattice: np.ndarray = None,
+    return_probe: bool = False,
 ) -> Union[np.ndarray, List[np.ndarray], Dict[Tuple[int], np.ndarray]]:
     """
     Generate a dynamical CBED pattern using the Bloch wave method.
@@ -463,12 +515,14 @@ def generate_CBED(
                                         the computation of the projected crystallographic directions
                                         becomes ambiguous. In this case, you must specify the indices of
                                         the zone axis used to generate the beams.
+        return_probe (bool):            If True, the probe (np.ndarray) will be returned in additon to the CBED
 
     Returns:
         If thickness is a scalar: CBED pattern as np.ndarray
         If thickness is a sequence: CBED patterns for each thickness value as a list of np.ndarrays
         If LACBED is True and thickness is scalar: Dictionary with tuples of ints (h,k,l) as keys, mapping to np.ndarray.
         If LACBED is True and thickness is a sequence: List of dictionaries, structured as above.
+        If return_probe is True: will return a tuple (<CBED/LACBED object>, Probe)
     """
 
     alpha_rad = alpha_mrad / 1000.0
@@ -481,12 +535,14 @@ def generate_CBED(
 
     # If there are only two beams, augment the list with a third perpendicular spot
     if qxy.shape[0] == 2:
-        assert two_beam_zone_axis_lattice is not None, "When only two beams are present, two_beam_zone_axis_lattice must be specified."
-        hkl_reflection = hkl[1] if np.all(qxy[0]==0.) else hkl[0]
-        qxy_reflection = qxy[1] if np.all(qxy[0]==0.) else qxy[0]
-        orthogonal_spot = np.cross(two_beam_zone_axis_lattice,hkl_reflection)
-        hkl_augmented = np.vstack((hkl,orthogonal_spot))
-        qxy_augmented = np.vstack((qxy,np.flipud(qxy_reflection)))
+        assert (
+            two_beam_zone_axis_lattice is not None
+        ), "When only two beams are present, two_beam_zone_axis_lattice must be specified."
+        hkl_reflection = hkl[1] if np.all(qxy[0] == 0.0) else hkl[0]
+        qxy_reflection = qxy[1] if np.all(qxy[0] == 0.0) else qxy[0]
+        orthogonal_spot = np.cross(two_beam_zone_axis_lattice, hkl_reflection)
+        hkl_augmented = np.vstack((hkl, orthogonal_spot))
+        qxy_augmented = np.vstack((qxy, np.flipud(qxy_reflection)))
         proj = np.linalg.lstsq(qxy_augmented, hkl_augmented, rcond=-1)[0]
         hkl_proj_x = proj[0] / np.linalg.norm(proj[0])
     # Otherwise calculate them based on the pattern
@@ -495,12 +551,20 @@ def generate_CBED(
         hkl_proj_x = proj[0] / np.linalg.norm(proj[0])
 
     # get unit vector in zone axis direction and projected x and y Cartesian directions:
-    zone_axis_rotation_matrix = self.parse_orientation(zone_axis_lattice=zone_axis_lattice,
-                                       zone_axis_cartesian=zone_axis_cartesian,
-                                       proj_x_lattice=hkl_proj_x)
-    ZA = np.array(zone_axis_rotation_matrix[:,2]) / np.linalg.norm(np.array(zone_axis_rotation_matrix[:,2]))
-    proj_x = zone_axis_rotation_matrix[:,0] / np.linalg.norm(zone_axis_rotation_matrix[:,0])
-    proj_y = zone_axis_rotation_matrix[:,1] / np.linalg.norm(zone_axis_rotation_matrix[:,1])
+    zone_axis_rotation_matrix = self.parse_orientation(
+        zone_axis_lattice=zone_axis_lattice,
+        zone_axis_cartesian=zone_axis_cartesian,
+        proj_x_lattice=hkl_proj_x,
+    )
+    ZA = np.array(zone_axis_rotation_matrix[:, 2]) / np.linalg.norm(
+        np.array(zone_axis_rotation_matrix[:, 2])
+    )
+    proj_x = zone_axis_rotation_matrix[:, 0] / np.linalg.norm(
+        zone_axis_rotation_matrix[:, 0]
+    )
+    proj_y = zone_axis_rotation_matrix[:, 1] / np.linalg.norm(
+        zone_axis_rotation_matrix[:, 1]
+    )
 
     # the foil normal should be the zone axis if unspecified
     if foil_normal_lattice is None:
@@ -573,6 +637,9 @@ def generate_CBED(
         # In CBED mode, the DP datastructure is a list of arrays
         DP = [np.zeros(DP_size, dtype=dtype) for _ in range(len(thickness))]
 
+    if return_probe:
+        probe = np.zeros(DP_size, dtype=dtype)
+
     mask = np.zeros(DP_size, dtype=np.bool_)
 
     Ugmh_cache = DynamicalMatrixCache()
@@ -587,6 +654,8 @@ def generate_CBED(
             always_return_list=True,
             dynamical_matrix_cache=Ugmh_cache,
         )
+        if return_probe:
+            probe[tx_pixels[i] + qx0, ty_pixels[i] + qy0] = 1
 
         if LACBED:
             # loop over each thickness
@@ -616,7 +685,13 @@ def generate_CBED(
             for patt, sim in zip(DP, bloch):
                 patt[xpix, ypix] += sim.data["intensity"][keep_mask]
 
-    if return_mask:
-        return (DP[0], mask) if len(thickness) == 1 else (DP, mask)
+    if not return_probe:
+        if return_mask:
+            return (DP[0], mask) if len(thickness) == 1 else (DP, mask)
+        else:
+            return DP[0] if len(thickness) == 1 else DP
     else:
-        return DP[0] if len(thickness) == 1 else DP
+        if return_mask:
+            return (DP[0], probe, mask) if len(thickness) == 1 else (DP, probe, mask)
+        else:
+            return (DP[0], probe) if len(thickness) == 1 else (DP, probe)
