@@ -11,7 +11,7 @@ from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar
 import matplotlib.font_manager as fm
 
 from .multicorr import upsampled_correlation
-from .tqdmnd import tqdmnd
+from ...utils.tqdmnd import tqdmnd
 
 try:
     from IPython.display import clear_output
@@ -19,6 +19,36 @@ except ImportError:
     def clear_output(wait=True):
         pass
 
+def radial_reduction(
+    ar,
+    x0,
+    y0,
+    binsize=1,
+    fn=np.mean,
+    coords = None):
+    """
+    Evaluate a reduction function on pixels within annular rings centered on (x0,y0),
+    with a ring width of binsize.
+
+    By default, returns the mean value of pixels within each annulus.
+    Some other useful reductions include: np.sum, np.std, np.count, np.median, ...
+
+    When running in a loop, pre-compute the pixel coordinates and pass them in
+    for improved performance, like so:
+        coords = np.mgrid[0:ar.shape[0],0:ar.shape[1]]
+        radial_sums = radial_reduction(ar, x0,y0, coords=coords)
+    """
+    qx,qy = coords if coords else np.mgrid[0:ar.shape[0],0:ar.shape[1]]
+    
+    r = np.floor(np.hypot(qx-x0,qy-y0).ravel()/binsize).astype(np.int64) * binsize
+    edges = np.cumsum(np.bincount(r)[::binsize])
+    slices = [slice(0,edges[0])] + [ slice(edges[i], edges[i+1]) for i in range(len(edges)-1)]
+    rargsort = np.argsort(r)
+    sorted_ar = ar.ravel()[rargsort]
+    reductions = np.array([fn(sorted_ar[s]) for s in slices])
+    
+    return reductions
+
 def plot(img, title='Image', savePath=None, cmap='inferno', show=True, vmax=None,
                                                         figsize=(10, 10), scale=None):
     fig, ax = plt.subplots(figsize=figsize)
@@ -130,49 +160,27 @@ def make_Fourier_coords2D(Nx, Ny, pixelSize=1):
     return qx, qy
 
 
-def get_shift(ar1, ar2, corrPower=1):
-    """
-	Determine the relative shift between a pair of identical arrays, or the shift giving
-	best overlap.
-
-	Shift determination uses the brightest pixel in the cross correlation, and is thus limited to
-    pixel resolution. corrPower specifies the cross correlation power, with 1 corresponding to a
-    cross correlation and 0 a phase correlation.
-
-	Args:
-		ar1,ar2 (2D ndarrays):
-        corrPower (float between 0 and 1, inclusive): 1=cross correlation, 0=phase
-            correlation
-
-    Returns:
-		(2-tuple): (shiftx,shifty) - the relative image shift, in pixels
-    """
-    cc = get_cross_correlation(ar1, ar2, corrPower)
-    xshift, yshift = np.unravel_index(np.argmax(cc), ar1.shape)
-    return xshift, yshift
-
-
 def get_shifted_ar(
-    ar, 
-    xshift, 
+    ar,
+    xshift,
     yshift,
     periodic=True,
     bilinear=False,
     ):
     """
-	Shifts array ar by the shift vector (xshift,yshift), using the either 
+	Shifts array ar by the shift vector (xshift,yshift), using the either
     the Fourier shift theorem (i.e. with sinc interpolation), or bilinear
     resampling. Boundary conditions can be periodic or not.
 
     Args:
-            ar (float): input array 
+            ar (float): input array
             xshift (float): shift along axis 0 (x) in pixels
             yshift (float): shift along axis 1 (y) in pixels
             periodic (bool): flag for periodic boundary conditions
             bilinear (bool): flag for bilinear image shifts
 
         Returns:
-            (array) the shifted array              
+            (array) the shifted array
     """
 
     # Apply image shift
@@ -189,7 +197,7 @@ def get_shifted_ar(
         yF = (np.floor(yshift)).astype(int)
         wx = xshift - xF
         wy = yshift - yF
-            
+
         shifted_ar = \
             np.roll(ar,(xF  ,yF  ),axis=(0,1)) * ((1-wx)*(1-wy)) + \
             np.roll(ar,(xF+1,yF  ),axis=(0,1)) * ((  wx)*(1-wy)) + \
@@ -204,49 +212,19 @@ def get_shifted_ar(
         if xR > 0:
             shifted_ar[0:xR,:] = 0
         elif xR < 0:
-            shifted_ar[xR:,:] = 0    
+            shifted_ar[xR:,:] = 0
         if yR > 0:
             shifted_ar[:,0:yR] = 0
         elif yR < 0:
-            shifted_ar[:,yR:] = 0    
+            shifted_ar[:,yR:] = 0
 
     return shifted_ar
 
 
-def get_cross_correlation(ar, kernel, corrPower=1, returnval='cc'):
-    """
-    Calculates the cross correlation of ar with kernel.
-    corrPower specifies the correlation type, where 1 is a cross correlation, 0 is a phase
-    correlation, and values in between are hybrids.
-
-    The return value depends on the argument ``returnval``. If return=='cc' (default),
-    returns the real part of the cross correlation in real space.  If return=='fourier',
-    returns the output in Fourier space, before taking the inverse transform.
-    """
-    assert(returnval in ('cc','fourier'))
-    fourierkernel = np.conj(np.fft.fft2(kernel))
-    return get_cross_correlation_fk(ar, fourierkernel, corrPower=corrPower, returnval=returnval)
 
 
-def get_cross_correlation_fk(ar, fourierkernel, corrPower=1, returnval='cc'):
-    """
-    Calculates the cross correlation of ar with fourierkernel.
-    Here, fourierkernel = np.conj(np.fft.fft2(kernel)); speeds up computation when the same
-    kernel is to be used for multiple cross correlations.
-    corrPower specifies the correlation type, where 1 is a cross correlation, 0 is a phase
-    correlation, and values in between are hybrids.
 
-    The return value depends on the argument ``returnval``. If return=='cc' (default),
-    returns the real part of the cross correlation in real space.  If return=='fourier',
-    returns the output in Fourier space, before taking the inverse transform.
-    """
-    assert(returnval in ('cc','fourier'))
-    m = np.fft.fft2(ar) * fourierkernel
-    ccc = np.abs(m)**(corrPower) * np.exp(1j*np.angle(m))
-    if returnval=='fourier':
-        return ccc
-    else:
-        return np.real(np.fft.ifft2(ccc))
+
 
 
 def get_CoM(ar):
@@ -261,167 +239,6 @@ def get_CoM(ar):
     return xCoM, yCoM
 
 
-def get_maximal_points(ar):
-    """
-    For 2D array ar, returns an array of bools of the same shape which is True for all entries with
-    values larger than all 8 of their nearest neighbors.  Excludes all boundary pixels.
-    """
-
-    # local max including periodic wrap-around
-    local_max = \
-        (ar >= np.roll(ar, (-1, 0), axis=(0, 1))) & (ar > np.roll(ar, (1, 0), axis=(0, 1))) & \
-        (ar >= np.roll(ar, (0, -1), axis=(0, 1))) & (ar > np.roll(ar, (0, 1), axis=(0, 1))) & \
-        (ar >= np.roll(ar, (-1, -1), axis=(0, 1))) & (ar > np.roll(ar, (-1, 1), axis=(0, 1))) & \
-        (ar >= np.roll(ar, (1, -1), axis=(0, 1))) & (ar > np.roll(ar, (1, 1), axis=(0, 1)))
-
-    # remove boundary max
-    local_max[0,:] = False
-    local_max[:,0] = False
-    local_max[-1,:] = False
-    local_max[:,-1] = False
-
-    return local_max
-
-
-def get_maxima_2D(ar,
-                  sigma=0,
-                  edgeBoundary=0,
-                  minSpacing=0,
-                  minRelativeIntensity=0,
-                  minAbsoluteIntensity=0,
-                  relativeToPeak=0,
-                  maxNumPeaks=1,
-                  subpixel='poly',
-                  ar_FT=None,
-                  upsample_factor=16):
-    """
-    Finds the indices where the 2D array ar is a local maximum.
-    Optional parameters allow blurring of the array and filtering of the output;
-    setting each of these to 0 (default) turns off these functions.
-
-    Args:
-        ar (ndarray): a 2D array
-        sigma (float): guassian blur std to applyu to ar before finding the maxima
-        edgeBoundary (int): ignore maxima within edgeBoundary of the array edge
-        minSpacing (float): if two maxima are found within minSpacing, the dimmer one
-            is removed
-        minRelativeIntensity (float): maxima dimmer than minRelativeIntensity compared
-            to the relativeToPeak'th brightest maximum are removed
-        minAbsoluteIntensity (float): the minimum acceptable correlation peak intensity,
-            on an absolute scale
-        relativeToPeak (int): 0=brightest maximum. 1=next brightest, etc.
-        maxNumPeaks (int): return only the first maxNumPeaks maxima. Set to None to return all peaks.
-        subpixel (str): Whether to use subpixel fitting, and which algorithm to use.
-            Must be in ('none','poly','multicorr').
-                * 'none': performs no subpixel fitting
-                * 'poly': polynomial interpolation of correlogram peaks (default)
-                * 'multicorr': uses the multicorr algorithm with DFT upsampling
-        ar_FT (None or complex array): if subpixel=='multicorr' the fourier transform of
-            the image is required.  It may be passed here as a complex array.  Otherwise,
-            if ar_FT is None, it is computed
-        upsample_factor (int): required iff subpixel=='multicorr'
-
-    Returns:
-        (3-tuple): A 3-tuple containing:
-
-            * **maxima_x** *(ndarray)*: x-coords of the local maximum, sorted by intensity.
-            * **maxima_y** *(ndarray)*: y-coords of the local maximum, sorted by intensity.
-            * **maxima_intensity** *(ndarray)*: intensity of the local maxima
-    """
-    assert subpixel in [ 'none', 'poly', 'multicorr' ], "Unrecognized subpixel option {}, subpixel must be 'none', 'poly', or 'multicorr'".format(subpixel)
-
-
-    if maxNumPeaks is not None:
-        maxNumPeaks = np.asarray(maxNumPeaks, dtype='int')
-
-    # Get maxima
-    ar = gaussian_filter(ar, sigma)
-    maxima_bool = get_maximal_points(ar)
-
-    # Remove edges
-    if edgeBoundary > 0:
-        assert isinstance(edgeBoundary, (int, np.integer))
-        maxima_bool[:edgeBoundary, :] = False
-        maxima_bool[-edgeBoundary:, :] = False
-        maxima_bool[:, :edgeBoundary] = False
-        maxima_bool[:, -edgeBoundary:] = False
-    elif subpixel is True:
-        maxima_bool[:1, :] = False
-        maxima_bool[-1:, :] = False
-        maxima_bool[:, :1] = False
-        maxima_bool[:, -1:] = False
-
-    # Get indices, sorted by intensity
-    maxima_x, maxima_y = np.nonzero(maxima_bool)
-    dtype = np.dtype([('x', float), ('y', float), ('intensity', float)])
-    maxima = np.zeros(len(maxima_x), dtype=dtype)
-    maxima['x'] = maxima_x
-    maxima['y'] = maxima_y
-    maxima['intensity'] = ar[maxima_x, maxima_y]
-    maxima = np.sort(maxima, order='intensity')[::-1]
-
-
-    if len(maxima) > 0:
-        # Remove maxima which are too close
-        if minSpacing > 0:
-            deletemask = np.zeros(len(maxima), dtype=bool)
-            for i in range(len(maxima)):
-                if deletemask[i] == False:
-                    tooClose = ((maxima['x'] - maxima['x'][i]) ** 2 + \
-                                (maxima['y'] - maxima['y'][i]) ** 2) < minSpacing ** 2
-                    tooClose[:i + 1] = False
-                    deletemask[tooClose] = True
-            maxima = np.delete(maxima, np.nonzero(deletemask)[0])
-
-        # Remove maxima which are too dim, versus the n-th brightest
-        if (minRelativeIntensity > 0) & (len(maxima) > relativeToPeak):
-            assert isinstance(relativeToPeak, (int, np.integer))
-            deletemask = maxima['intensity'] / maxima['intensity'][relativeToPeak] < minRelativeIntensity
-            maxima = np.delete(maxima, np.nonzero(deletemask)[0])
-
-        # Remove maxima which are too dim, absolute scale
-        if (minAbsoluteIntensity > 0):
-            deletemask = maxima['intensity'] < minAbsoluteIntensity
-            maxima = np.delete(maxima, np.nonzero(deletemask)[0])
-
-        # Remove maxima in excess of maxNumPeaks
-        if maxNumPeaks is not None:
-            if len(maxima) > maxNumPeaks:
-                maxima = maxima[:maxNumPeaks]
-
-
-        # Subpixel fitting 
-        # For all subpixel fitting, first fit 1D parabolas in x and y to 3 points (maximum, +/- 1 pixel)
-        if subpixel != 'none':
-            for i in range(len(maxima)):
-                Ix1_ = ar[int(maxima['x'][i]) - 1, int(maxima['y'][i])].astype(np.float)
-                Ix0 = ar[int(maxima['x'][i]), int(maxima['y'][i])].astype(np.float)
-                Ix1 = ar[int(maxima['x'][i]) + 1, int(maxima['y'][i])].astype(np.float)
-                Iy1_ = ar[int(maxima['x'][i]), int(maxima['y'][i]) - 1].astype(np.float)
-                Iy0 = ar[int(maxima['x'][i]), int(maxima['y'][i])].astype(np.float)
-                Iy1 = ar[int(maxima['x'][i]), int(maxima['y'][i]) + 1].astype(np.float)
-                deltax = (Ix1 - Ix1_) / (4 * Ix0 - 2 * Ix1 - 2 * Ix1_)
-                deltay = (Iy1 - Iy1_) / (4 * Iy0 - 2 * Iy1 - 2 * Iy1_)
-                maxima['x'][i] += deltax
-                maxima['y'][i] += deltay
-                maxima['intensity'][i] = linear_interpolation_2D(ar, maxima['x'][i], maxima['y'][i])
-        # Further refinement with fourier upsampling
-        if subpixel == 'multicorr':
-            if ar_FT is None:
-                ar_FT = np.fft.fft2(ar)
-            for ipeak in range(len(maxima['x'])):
-                xyShift = np.array((maxima['x'][ipeak],maxima['y'][ipeak]))
-                # we actually have to lose some precision and go down to half-pixel
-                # accuracy. this could also be done by a single upsampling at factor 2
-                # instead of get_maxima_2D.
-                xyShift[0] = np.round(xyShift[0] * 2) / 2
-                xyShift[1] = np.round(xyShift[1] * 2) / 2
-
-                subShift = upsampled_correlation(ar_FT,upsample_factor,xyShift)
-                maxima['x'][ipeak]=subShift[0]
-                maxima['y'][ipeak]=subShift[1]
-
-    return maxima['x'], maxima['y'], maxima['intensity']
 
 
 def get_maxima_1D(ar, sigma=0, minSpacing=0, minRelativeIntensity=0, relativeToPeak=0):
@@ -486,18 +303,6 @@ def linear_interpolation_1D(ar, x):
     return (1 - dx) * ar[x0] + dx * ar[x1]
 
 
-def linear_interpolation_2D(ar, x, y):
-    """
-    Calculates the 2D linear interpolation of array ar at position x,y using the four
-    nearest array elements.
-    """
-    x0, x1 = int(np.floor(x)), int(np.ceil(x))
-    y0, y1 = int(np.floor(y)), int(np.ceil(y))
-    dx = x - x0
-    dy = y - y0
-    return (1 - dx) * (1 - dy) * ar[x0, y0] + (1 - dx) * dy * ar[x0, y1] + dx * (1 - dy) * ar[x1, y0] + dx * dy * ar[
-        x1, y1]
-
 
 def add_to_2D_array_from_floats(ar, x, y, I):
     """
@@ -519,31 +324,6 @@ def add_to_2D_array_from_floats(ar, x, y, I):
     ar[x1[mask], y1[mask]] += (    dx[mask]) * (    dy[mask]) * I[mask]
     return ar
 
-
-def print_progress_bar(iteration, total, prefix='', suffix='', decimals=1,
-                       length=100, fill='*'):
-    """
-    Call in a loop to create terminal progress bar
-
-    Args:
-        iteration (int): current iteration
-        total (int): total iterations
-        prefix (str, optional): prefix string
-        suffix  (str, optional): suffix string
-        decimals (int, optional): positive number of decimals in percent complete
-        length (int, optional): character length of bar
-        fill (str, optional): bar fill character
-    """
-    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
-    filledLength = int(length * iteration // total)
-    bar = fill * filledLength + '-' * (length - filledLength)
-    clear_output(wait=True)
-    print('\r{} |{}| {}% {}\r'.format(prefix, bar, percent, suffix))
-    # Print New Line on Complete
-    if iteration == total:
-        print()
-
-
 def bin2D(array, factor, dtype=np.float64):
     """
     Bin a 2D ndarray by binfactor.
@@ -693,24 +473,24 @@ def get_ewpc_filter_function(Q_Nx, Q_Ny):
 
 
 def fourier_resample(
-    array, 
-    scale=None, 
+    array,
+    scale=None,
     output_size=None,
     force_nonnegative=False,
     bandlimit_nyquist=None,
-    bandlimit_power=2, 
+    bandlimit_power=2,
     dtype=np.float32):
     """
     Resize a 2D array along any dimension, using Fourier interpolation / extrapolation.
     For 4D input arrays, only the final two axes can be resized.
 
     The scaling of the array can be specified by passing either `scale`, which sets
-    the scaling factor along both axes to be scaled; or by passing `output_size`, 
+    the scaling factor along both axes to be scaled; or by passing `output_size`,
     which specifies the final dimensions of the scaled axes (and allows for different
     scaling along the x,y or kx,ky axes.)
 
     Args:
-        array (2D/4D numpy array): Input array, or 4D stack of arrays, to be resized. 
+        array (2D/4D numpy array): Input array, or 4D stack of arrays, to be resized.
         scale (float): scalar value giving the scaling factor for all dimensions
         output_size (2-tuple of ints): two values giving either the (x,y) output size for 2D, or (kx,ky) for 4D
         force_nonnegative (bool): Force all outputs to be nonnegative, after filtering
@@ -740,7 +520,7 @@ def fourier_resample(
         assert scale is None, 'Cannot specify both a scaling factor and output size'
         assert np.size(output_size) == 2, 'output_size must contain two values'
         output_size = np.asarray(output_size)
-    
+
     scale_output = np.prod(output_size) / np.prod(input__size)
 
 
@@ -901,8 +681,57 @@ def fourier_resample(
     # Enforce positivity if needed, after filtering
     if force_nonnegative:
         array_resize = np.maximum(array_resize,0)
-        
+
     # Normalization
     array_resize = array_resize * scale_output
 
     return array_resize
+
+
+
+
+
+
+#import matplotlib.pyplot as plt
+#from mpl_toolkits.axes_grid1 import make_axes_locatable
+#from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar
+#import matplotlib.font_manager as fm
+#
+#
+#try:
+#    from IPython.display import clear_output
+#except ImportError:
+#    def clear_output(wait=True):
+#        pass
+#
+#def plot(img, title='Image', savePath=None, cmap='inferno', show=True, vmax=None,
+#                                                        figsize=(10, 10), scale=None):
+#    fig, ax = plt.subplots(figsize=figsize)
+#    im = ax.imshow(img, interpolation='nearest', cmap=plt.cm.get_cmap(cmap), vmax=vmax)
+#    divider = make_axes_locatable(ax)
+#    cax = divider.append_axes("right", size="5%", pad=0.05)
+#    plt.colorbar(im, cax=cax)
+#    ax.set_title(title)
+#    fontprops = fm.FontProperties(size=18)
+#    if scale is not None:
+#        scalebar = AnchoredSizeBar(ax.transData,
+#                                   scale[0], scale[1], 'lower right',
+#                                   pad=0.1,
+#                                   color='white',
+#                                   frameon=False,
+#                                   size_vertical=img.shape[0] / 40,
+#                                   fontproperties=fontprops)
+#
+#        ax.add_artist(scalebar)
+#    ax.grid(False)
+#    if savePath is not None:
+#        fig.savefig(savePath + '.png', dpi=600)
+#        fig.savefig(savePath + '.eps', dpi=600)
+#    if show:
+#        plt.show()
+
+
+
+
+
+
